@@ -1,33 +1,24 @@
-from typing import List, NewType
+from typing import List, Set
 
 from keycloak_scanner.logging.vuln_flag import VulnFlag
-from keycloak_scanner.scanners.json_result import JsonResult
-from keycloak_scanner.scanners.scanner import Scanner
+from keycloak_scanner.scan_base.scanner import Scanner
+from keycloak_scanner.scan_base.types import Realm
+from keycloak_scanner.scan_base.wrap import WrapperTypes
 
 URL_PATTERN = '{}/auth/realms/{}'
 
 
-class Realm(JsonResult):
-    pass
-
-
-class Realms(List[Realm]):
-    pass
-
-
-class RealmScanner(Scanner[Realms]):
-
-    DEFAULT_REALMS = ['master']
+class RealmScanner(Scanner[Realm]):
 
     def __init__(self, realms: List[str] = None, **kwargs):
         if realms is None:
-            realms = RealmScanner.DEFAULT_REALMS
+            realms = []
         self.realms = realms
-        super().__init__(**kwargs)
+        super().__init__(result_type=WrapperTypes.REALM_TYPE, **kwargs)
 
-    def perform(self) -> (Realms, VulnFlag):
+    def perform(self) -> (Set[Realm], VulnFlag):
 
-        realms: Realms = Realms([])
+        realms: Set[Realm] = set()
 
         for realm_name in self.realms:
 
@@ -43,6 +34,6 @@ class RealmScanner(Scanner[Realms]):
 
                 if 'public_key' in realm.json:
                     super().info(f'Public key for realm {realm_name} : {realm.json["public_key"]}')
-                realms.append(realm)
+                realms.add(realm)
 
         return realms, VulnFlag()
