@@ -1,20 +1,15 @@
-from unittest.mock import MagicMock
+from pathlib import Path
+from typing import List
 
-import requests
 from _pytest.fixtures import fixture
 from requests import Session
-import logging
-import http.client
 
-from keycloak_scanner.logging.http_logging import httpclient_logging_patch
-from keycloak_scanner.scanners.clients_scanner import Client, Clients
-from keycloak_scanner.scanners.realm_scanner import Realm, Realms
-from keycloak_scanner.scanners.security_console_scanner import SecurityConsoleResults, SecurityConsoleResult
-from keycloak_scanner.scanners.well_known_scanner import WellKnownDict, WellKnown
-from tests.mock_response import MockResponse
+from keycloak_scanner.scan_base.types import Realm, SecurityConsole, WellKnown, Client
+
+from tests.mock_response import MockResponse, RequestSpec, MockSpec
 
 
-#httpclient_logging_patch()
+# httpclient_logging_patch()
 
 
 @fixture
@@ -138,18 +133,25 @@ def well_known_json_other() -> dict:
             "backchannel_authentication_endpoint": "http://localhost:8080/auth/realms/other/protocol/openid-connect/ext/ciba/auth"}
 
 
-@fixture()
-def well_known_dict(master_realm: Realm, other_realm: Realm, well_known_json_master: dict,
-                    well_known_json_other: dict) -> WellKnownDict:
+
+@fixture
+def well_known_master(master_realm: Realm, well_known_json_master: dict) -> WellKnown:
+    return WellKnown(realm=master_realm, name='master',
+                      url='http://localhost:8080/auth/realms/master/.well-known/openid-configuration',
+                      json=well_known_json_master)
+
+
+@fixture
+def well_known_other(other_realm: Realm, well_known_json_other: dict) -> WellKnown:
+    return WellKnown(realm=other_realm, name='other',
+                      url='http://localhost:8080/auth/realms/other/.well-known/openid-configuration',
+                      json=well_known_json_other)
+
+
+@fixture
+def well_known_list(well_known_master: WellKnown, well_known_other: WellKnown) -> List[WellKnown]:
     # TODO: master wk json in all
-    return WellKnownDict({
-        'master': WellKnown(realm=master_realm, name='master',
-                            url='http://localhost:8080/auth/realms/master/.well-known/openid-configuration',
-                            json=well_known_json_master),
-        'other': WellKnown(realm=other_realm, name='other',
-                           url='http://localhost:8080/auth/realms/other/.well-known/openid-configuration',
-                           json=well_known_json_other)
-    })
+    return [well_known_master, well_known_other]
 
 
 @fixture
@@ -174,8 +176,8 @@ def other_realm_json() -> dict:
 
 
 @fixture
-def all_realms(master_realm: Realm, other_realm: Realm) -> Realms:
-    return Realms([master_realm, other_realm])
+def all_realms(master_realm: Realm, other_realm: Realm) -> List[Realm]:
+    return [master_realm, other_realm]
 
 
 @fixture
@@ -185,32 +187,31 @@ def other_realm(other_realm_json: dict) -> Realm:
 
 @fixture
 def client1() -> Client:
-    return Client(name='client1', url='http://localhost:8080/auth/realms/master/client1',
-                  auth_endpoint='http://localhost:8080/auth/realms/master/protocol/openid-connect/auth')
+    return Client(name='client1', url='http://localhost:8080/auth/realms/master/client1')
 
 
 @fixture
 def client2() -> Client:
-    return Client(name='client2', url='http://localhost:8080/auth/realms/master/client2',
-                  auth_endpoint='http://localhost:8080/auth/realms/master/protocol/openid-connect/auth')
+    return Client(name='client2', url='http://localhost:8080/auth/realms/master/client2')
 
 
 @fixture
-def all_clients(client1: Client, client2: Client) -> Clients:
-    return Clients([client1, client2])
+def all_clients(client1: Client, client2: Client) -> List[Client]:
+    return [client1, client2]
 
 
 @fixture
-def security_console_results(master_realm: Realm, other_realm: Realm) -> SecurityConsoleResults:
-    return SecurityConsoleResults({
-        'master': SecurityConsoleResult(master_realm,
+def security_console_results(master_realm: Realm, other_realm: Realm) -> List[SecurityConsole]:
+    return [
+         SecurityConsole(master_realm,
                                         'http://localhost:8080/auth/realms/master/clients-registrations/default/security-admin-console',
                                         json={}),
 
-        'other': SecurityConsoleResult(other_realm,
+         SecurityConsole(other_realm,
                                        'http://localhost:8080/auth/realms/other/clients-registrations/default/security-admin-console',
                                        json={}, secret={'secret': 'secretdata'}),
-    })
+    ]
+
 
 @fixture
 def login_html_page():
@@ -302,62 +303,150 @@ def login_html_page():
     '''
 
 
+@fixture
+def full_scan_mock(master_realm_json, other_realm_json, well_known_json_master: dict,
+                   well_known_json_other: dict, login_html_page: str) -> MockSpec:
+    token_response = {
+        'access_token': 'eyJhbGciOiJIUzI1NiIsInR5cCIgOiAiSldUIiwia2lkIiA6ICI3ODM4MGM2ZS1iODhmLTQ5NDQtOGRkZS03NTQyMDNkMjFhODEifQ.eyJleHAiOjE2MjE2NzU5NzIsImlhdCI6MTYyMTYzOTk3MiwianRpIjoiMGU2NDcxOTItMzU5ZS00NmU4LWFkYWQtNTQzNmQyNjMyZjA1IiwiaXNzIjoiaHR0cDovL2xvY2FsaG9zdDo4MDgwL2F1dGgvcmVhbG1zL21hc3RlciIsInN1YiI6IjJjMTZhY2Y1LWMwOTYtNDg5ZC1iYjFjLTU4ZTc0ZTJiZjAzMiIsInR5cCI6IlNlcmlhbGl6ZWQtSUQiLCJzZXNzaW9uX3N0YXRlIjoiZWY3ZjNjZmItMDAzZS00YzViLWEzMWQtYmI0OGFhZjAzNzk3Iiwic3RhdGVfY2hlY2tlciI6ImtKNy05MURtNVEwVXktT1JfVlJnT1d5eF91Wkh3M0ZfczktTVdlUjZRTlEifQ.6yZvyGKEH0NXmLY8nKRQMLsMQYPXq5dYCsIF3LRiOxI',
+        'refresh_token': 'eyJhbGciOiJIUzI1NiIsInR5cCIgOiAiSldUIiwia2lkIiA6ICI3ODM4MGM2ZS1iODhmLTQ5NDQtOGRkZS03NTQyMDNkMjFhODEifQ.eyJleHAiOjE2MjE2NzU5NzIsImlhdCI6MTYyMTYzOTk3MiwianRpIjoiMGU2NDcxOTItMzU5ZS00NmU4LWFkYWQtNTQzNmQyNjMyZjA1IiwiaXNzIjoiaHR0cDovL2xvY2FsaG9zdDo4MDgwL2F1dGgvcmVhbG1zL21hc3RlciIsInN1YiI6IjJjMTZhY2Y1LWMwOTYtNDg5ZC1iYjFjLTU4ZTc0ZTJiZjAzMiIsInR5cCI6IlNlcmlhbGl6ZWQtSUQiLCJzZXNzaW9uX3N0YXRlIjoiZWY3ZjNjZmItMDAzZS00YzViLWEzMWQtYmI0OGFhZjAzNzk3Iiwic3RhdGVfY2hlY2tlciI6ImtKNy05MURtNVEwVXktT1JfVlJnT1d5eF91Wkh3M0ZfczktTVdlUjZRTlEifQ.6yZvyGKEH0NXmLY8nKRQMLsMQYPXq5dYCsIF3LRiOxI'
+    }
+
+    return MockSpec(get={
+        'http://localhost:8080/auth/realms/master/.well-known/openid-configuration': RequestSpec(
+            MockResponse(status_code=200, response=well_known_json_master)
+        ),
+        'http://localhost:8080/auth/realms/master': RequestSpec(
+            MockResponse(status_code=200, response=master_realm_json)),
+        'http://localhost:8080/auth/realms/other': RequestSpec(
+            MockResponse(status_code=200, response=other_realm_json)),
+        'http://localhost:8080/auth/realms/other/.well-known/openid-configuration': RequestSpec(
+            MockResponse(status_code=200,
+                         response=well_known_json_other)),
+        'http://localhost:8080/auth/realms/master/client1': RequestSpec(
+            MockResponse(status_code=200, response='coucou')),
+        'http://localhost:8080/auth/realms/master/client2': RequestSpec(
+            MockResponse(status_code=200, response='coucou')),
+        'http://localhost:8080/auth/realms/other/client1': RequestSpec(
+            MockResponse(status_code=200, response='coucou')),
+        'http://localhost:8080/auth/realms/other/client2': RequestSpec(
+            MockResponse(status_code=200, response='coucou')),
+        'http://localhost:8080/auth/realms/master/clients-registrations/default/security-admin-console':
+            RequestSpec(MockResponse(status_code=401, response={"error": "invalid_token",
+                                                                "error_description": "Not authorized to view client. Not valid token or client credentials provided."})),
+        'http://localhost:8080/auth/realms/other/clients-registrations/default/security-admin-console': RequestSpec(
+            MockResponse(
+                status_code=401, response={"error": "invalid_token",
+                                           "error_description": "Not authorized to view client. Not valid token or client credentials provided."})),
+        'http://localhost:8080/auth': RequestSpec(MockResponse(status_code=400)),
+        'http://localhost:8080/auth/realms/master/protocol/openid-connect/auth': RequestSpec(MockResponse(200,
+                                                                                                          response=login_html_page)),
+        'http://localhost:8080/auth/realms/other/protocol/openid-connect/auth': RequestSpec(MockResponse(200,
+                                                                                                         response=login_html_page)),
+        'http://localhost:8080/auth/realms/master/protocol/openid-connect/auth?client_id=account-console&redirect_uri=http%3A%2F%2Flocalhost%3A8080%2Fauth%2Frealms%2Fmaster%2Faccount%2F%23%2F&state=310f298c-f3d8-4c42-8ebc-44484febf84c&response_mode=fragment&response_type=code&scope=openid&nonce=a6be5274-15e4-4ffe-9905-ffb038b20a8e&code_challenge=Nd1svU3YNT0r6eWHkSmNeX_cxgUPQUVzPfZFXRWaJmY&code_challenge_method=S256':
+            RequestSpec(MockResponse(
+                200, login_html_page)),
+        'http://localhost:8080/realms/master/clients-registrations/default/client1': RequestSpec(
+            MockResponse(200, response={"id": "899e2dc1-5fc0-4eaf-bedb-f81a3f9e9313", "clientId": "admin-cli",
+                                        "name": "${client_admin-cli}", "surrogateAuthRequired": False, "enabled": True,
+                                        "alwaysDisplayInConsole": False, "clientAuthenticatorType": "client-secret",
+                                        "redirectUris": [], "webOrigins": [], "notBefore": 0, "bearerOnly": False,
+                                        "consentRequired": False, "standardFlowEnabled": False,
+                                        "implicitFlowEnabled": False, "directAccessGrantsEnabled": False,
+                                        "serviceAccountsEnabled": False, "publicClient": False,
+                                        "frontchannelLogout": False, "protocol": "openid-connect", "attributes": {},
+                                        "authenticationFlowBindingOverrides": {}, "fullScopeAllowed": False,
+                                        "nodeReRegistrationTimeout": 0,
+                                        "defaultClientScopes": ["web-origins", "roles", "profile", "email"],
+                                        "optionalClientScopes": ["address", "phone", "offline_access",
+                                                                 "microprofile-jwt"]})
+        ),
+        'http://localhost:8080/realms/other/clients-registrations/default/client1': RequestSpec(
+            MockResponse(200, response={"id": "899e2dc1-5fc0-4eaf-bedb-f81a3f9e9313", "clientId": "admin-cli",
+                                        "name": "${client_admin-cli}", "surrogateAuthRequired": False, "enabled": True,
+                                        "alwaysDisplayInConsole": False, "clientAuthenticatorType": "client-secret",
+                                        "redirectUris": [], "webOrigins": [], "notBefore": 0, "bearerOnly": False,
+                                        "consentRequired": False, "standardFlowEnabled": False,
+                                        "implicitFlowEnabled": False, "directAccessGrantsEnabled": False,
+                                        "serviceAccountsEnabled": False, "publicClient": False,
+                                        "frontchannelLogout": False, "protocol": "openid-connect", "attributes": {},
+                                        "authenticationFlowBindingOverrides": {}, "fullScopeAllowed": False,
+                                        "nodeReRegistrationTimeout": 0,
+                                        "defaultClientScopes": ["web-origins", "roles", "profile", "email"],
+                                        "optionalClientScopes": ["address", "phone", "offline_access",
+                                                                 "microprofile-jwt"]})
+        ),
+        'http://localhost:8080/realms/master/clients-registrations/default/client2': RequestSpec(
+            MockResponse(400)
+        ),
+        'http://localhost:8080/realms/other/clients-registrations/default/client2': RequestSpec(
+            MockResponse(400)
+        ),
+
+    },
+        post={
+            'http://localhost:8080/master/token': RequestSpec(MockResponse(status_code=200, response=token_response)),
+            'http://localhost:8080/auth/realms/master/protocol/openid-connect/token': RequestSpec(
+                MockResponse(status_code=200,
+                             response=token_response)),
+            'http://localhost:8080/other/token': RequestSpec(MockResponse(status_code=200, response=token_response)),
+            'http://localhost:8080/auth/realms/other/protocol/openid-connect/token': RequestSpec(
+                MockResponse(status_code=200,
+                             response=token_response)),
+            'http://localhost:8080/auth/realms/master/login-actions/authenticate?session_code'
+            '=bR4rBd0QNGsd_kGuqiyLEuYuY6FK3Lx9HCYJEltUQBk&execution=de13838a-ee3d-404e-b16d-b0d7aa320844&client_id'
+            '=account-console&tab_id=GXMjAPR3DsQ':
+                RequestSpec(MockResponse(
+                    302, response=None, headers={'Location': '<openid location>'})),
+            'http://localhost:8080/auth/realms/master/clients-registrations/openid-connect':
+                RequestSpec(response=MockResponse(status_code=201, response={
+                    "redirect_uris":
+                        ["http://localhost:8080/callback"],
+                    "token_endpoint_auth_method": "client_secret_basic",
+                    "grant_types": ["authorization_code", "refresh_token"],
+                    "response_types": ["code", "none"],
+                    "client_id": "539ce782-5d15-4256-a5fa-1a46609d056b",
+                    "client_secret": "c94f5fc0-0a04-4e2f-aec6-b1f5edad1d44",
+                    "client_name": "keycloak-client-456789",
+                    "scope": "address phone offline_access microprofile-jwt",
+                    "jwks_uri": "http://localhost:8080/public_keys.jwks",
+                    "subject_type": "pairwise",
+                    "request_uris": ["http://localhost:8080/rf.txt"],
+                    "tls_client_certificate_bound_access_tokens": False,
+                    "client_id_issued_at": 1622306364,
+                    "client_secret_expires_at": 0,
+                    "registration_client_uri": "http://localhost:8080/auth/realms/master/clients-registrations/openid-connect/539ce782-5d15-4256-a5fa-1a46609d056b",
+                    "backchannel_logout_session_required": False
+                })),
+            'http://localhost:8080/auth/realms/other/clients-registrations/openid-connect':
+                RequestSpec(response=MockResponse(status_code=201, response={
+                    "redirect_uris":
+                        ["http://localhost:8080/callback"],
+                    "token_endpoint_auth_method": "client_secret_basic",
+                    "grant_types": ["authorization_code", "refresh_token"],
+                    "response_types": ["code", "none"],
+                    "client_id": "539ce782-5d15-4256-a5fa-1a46609d056b",
+                    "client_secret": "c94f5fc0-0a04-4e2f-aec6-b1f5edad1d44",
+                    "client_name": "keycloak-client-456789",
+                    "scope": "address phone offline_access microprofile-jwt",
+                    "jwks_uri": "http://localhost:8080/public_keys.jwks",
+                    "subject_type": "pairwise",
+                    "request_uris": ["http://localhost:8080/rf.txt"],
+                    "tls_client_certificate_bound_access_tokens": False,
+                    "client_id_issued_at": 1622306364,
+                    "client_secret_expires_at": 0,
+                    "registration_client_uri": "http://localhost:8080/auth/realms/other/clients-registrations/openid-connect/539ce782-5d15-4256-a5fa-1a46609d056b",
+                    "backchannel_logout_session_required": False
+                }))
+        })
+
 
 @fixture
-def full_scan_mock_session(master_realm_json, other_realm_json, well_known_json_master: dict,
-                           well_known_json_other: dict, login_html_page: str) -> Session:
-    def get_mock_response(url, **kwargs):
-        responses = {
-            'http://localhost:8080/auth/realms/master/.well-known/openid-configuration': MockResponse(status_code=200,
-                                                                                                      response=well_known_json_master),
-            'http://localhost:8080/auth/realms/master': MockResponse(status_code=200, response=master_realm_json),
-            'http://localhost:8080/auth/realms/other': MockResponse(status_code=200, response=other_realm_json),
-            'http://localhost:8080/auth/realms/other/.well-known/openid-configuration': MockResponse(status_code=200,
-                                                                                                     response=well_known_json_other),
-            'http://localhost:8080/auth/realms/master/client1': MockResponse(status_code=200, response='coucou'),
-            'http://localhost:8080/auth/realms/master/client2': MockResponse(status_code=200, response='coucou'),
-            'http://localhost:8080/auth/realms/other/client1': MockResponse(status_code=200, response='coucou'),
-            'http://localhost:8080/auth/realms/other/client2': MockResponse(status_code=200, response='coucou'),
-            'http://localhost:8080/auth/realms/master/clients-registrations/default/security-admin-console': MockResponse(
-                status_code=401, response={"error": "invalid_token",
-                                           "error_description": "Not authorized to view client. Not valid token or client credentials provided."}),
-            'http://localhost:8080/auth/realms/other/clients-registrations/default/security-admin-console': MockResponse(
-                status_code=401, response={"error": "invalid_token",
-                                           "error_description": "Not authorized to view client. Not valid token or client credentials provided."}),
-            'http://localhost:8080/auth': MockResponse(status_code=400),
-            'http://localhost:8080/auth/realms/master/protocol/openid-connect/auth': MockResponse(200, response=login_html_page),
-            'http://localhost:8080/auth/realms/other/protocol/openid-connect/auth': MockResponse(200, response=login_html_page),
-            'http://localhost:8080/auth/realms/master/protocol/openid-connect/auth?client_id=account-console&redirect_uri=http%3A%2F%2Flocalhost%3A8080%2Fauth%2Frealms%2Fmaster%2Faccount%2F%23%2F&state=310f298c-f3d8-4c42-8ebc-44484febf84c&response_mode=fragment&response_type=code&scope=openid&nonce=a6be5274-15e4-4ffe-9905-ffb038b20a8e&code_challenge=Nd1svU3YNT0r6eWHkSmNeX_cxgUPQUVzPfZFXRWaJmY&code_challenge_method=S256': MockResponse(200, login_html_page)
-        }
-        if url not in responses:
-            raise Exception(f'bad url test (GET) : {url}')
-        return responses[url]
+def full_scan_mock_session(full_scan_mock: MockSpec) -> Session:
+    return full_scan_mock.session()
 
-    def post_mock_response(url, data=None, **kwargs):
-        if data is None:
-            data = {}
-        token_response = {
-            'access_token': 'eyJhbGciOiJIUzI1NiIsInR5cCIgOiAiSldUIiwia2lkIiA6ICI3ODM4MGM2ZS1iODhmLTQ5NDQtOGRkZS03NTQyMDNkMjFhODEifQ.eyJleHAiOjE2MjE2NzU5NzIsImlhdCI6MTYyMTYzOTk3MiwianRpIjoiMGU2NDcxOTItMzU5ZS00NmU4LWFkYWQtNTQzNmQyNjMyZjA1IiwiaXNzIjoiaHR0cDovL2xvY2FsaG9zdDo4MDgwL2F1dGgvcmVhbG1zL21hc3RlciIsInN1YiI6IjJjMTZhY2Y1LWMwOTYtNDg5ZC1iYjFjLTU4ZTc0ZTJiZjAzMiIsInR5cCI6IlNlcmlhbGl6ZWQtSUQiLCJzZXNzaW9uX3N0YXRlIjoiZWY3ZjNjZmItMDAzZS00YzViLWEzMWQtYmI0OGFhZjAzNzk3Iiwic3RhdGVfY2hlY2tlciI6ImtKNy05MURtNVEwVXktT1JfVlJnT1d5eF91Wkh3M0ZfczktTVdlUjZRTlEifQ.6yZvyGKEH0NXmLY8nKRQMLsMQYPXq5dYCsIF3LRiOxI',
-            'refresh_token': 'eyJhbGciOiJIUzI1NiIsInR5cCIgOiAiSldUIiwia2lkIiA6ICI3ODM4MGM2ZS1iODhmLTQ5NDQtOGRkZS03NTQyMDNkMjFhODEifQ.eyJleHAiOjE2MjE2NzU5NzIsImlhdCI6MTYyMTYzOTk3MiwianRpIjoiMGU2NDcxOTItMzU5ZS00NmU4LWFkYWQtNTQzNmQyNjMyZjA1IiwiaXNzIjoiaHR0cDovL2xvY2FsaG9zdDo4MDgwL2F1dGgvcmVhbG1zL21hc3RlciIsInN1YiI6IjJjMTZhY2Y1LWMwOTYtNDg5ZC1iYjFjLTU4ZTc0ZTJiZjAzMiIsInR5cCI6IlNlcmlhbGl6ZWQtSUQiLCJzZXNzaW9uX3N0YXRlIjoiZWY3ZjNjZmItMDAzZS00YzViLWEzMWQtYmI0OGFhZjAzNzk3Iiwic3RhdGVfY2hlY2tlciI6ImtKNy05MURtNVEwVXktT1JfVlJnT1d5eF91Wkh3M0ZfczktTVdlUjZRTlEifQ.6yZvyGKEH0NXmLY8nKRQMLsMQYPXq5dYCsIF3LRiOxI'
-        }
-        responses = {
-            'http://localhost:8080/master/token': MockResponse(status_code=200, response=token_response),
-            'http://localhost:8080/auth/realms/master/protocol/openid-connect/token': MockResponse(status_code=200,
-                                                                                                   response=token_response),
-            'http://localhost:8080/other/token': MockResponse(status_code=200, response=token_response),
-            'http://localhost:8080/auth/realms/other/protocol/openid-connect/token': MockResponse(status_code=200,
-                                                                                                  response=token_response),
-            'http://localhost:8080/auth/realms/master/login-actions/authenticate?session_code=bR4rBd0QNGsd_kGuqiyLEuYuY6FK3Lx9HCYJEltUQBk&execution=de13838a-ee3d-404e-b16d-b0d7aa320844&client_id=account-console&tab_id=GXMjAPR3DsQ': MockResponse(
-                302, response=None, headers={'Location': '<openid location>'})
-        }
-        if url not in responses:
-            raise Exception(f'bad url test (POST) : {url}')
-        return responses[url]
 
-    session = requests.Session()
-    session.get = MagicMock(side_effect=get_mock_response)
-    session.post = MagicMock(side_effect=post_mock_response)
-    session.put = MagicMock()
-    session.delete = MagicMock()
-
-    return session
+@fixture
+def callback_file(tmp_path: Path) -> Path:
+    p = tmp_path / 'callback.txt'
+    p.write_text('http://callback\nhttp://callback2\n')
+    return p
